package de.acidum.indexer.core;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URL;
import java.util.Date;

import org.apache.log4j.Logger;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.SimpleAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
//import org.apache.lucene.index.IndexWriter.MaxFieldLength;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;

import de.acidum.indexer.parser.ContentParser;
import de.acidum.indexer.parser.ContentParserException;
import de.acidum.indexer.tika.parser.TikaDocumentParser;

/**
 *
 * @author Christoph Hartmann
 *
 */
public class Indexer {

	static Logger logger = Logger.getLogger(Indexer.class);

	volatile protected ContentParser contentParser;
	Boolean contentParserAccess = true;

	public Indexer() throws IOException {

	}

	/**
	 * Index the given file and use the given writer to store it.
	 *
	 * @param writer
	 * @param file
	 * @throws ContentParserException
	 */
	public void index(IndexWriter writer, File file)
			throws ContentParserException {

		logger.debug(file.getAbsolutePath());

		if (contentParser == null)
			return;

		// iterate over folders and files
		if (file.canRead()) {
			if (file.isDirectory()) {
				String[] files = file.list();
				if (files != null) {
					for (int i = 0; i < files.length; i++) {
						try {
							index(writer, new File(file, files[i]));
						} catch (Exception e) {
							logger.error(e);
						}
					}
				}
			} else {
				logger.debug("Indexing " + file);
				try {
					Document doc = null;
					// parse the document
					synchronized (contentParserAccess) {
						doc = contentParser.getDocument(file);
					}

					// put it into Lucene
					if (doc != null) {
						writer.addDocument(doc);
					} else {
						logger.error("Cannot handle "
								+ file.getAbsolutePath() + "; skipping");
					}
				} catch (IOException e) {
					logger.error("Cannot index " + file.getAbsolutePath()
							+ "; skipping (" + e.getMessage() + ")");
				}
			}
		}
	}

	/**
	 * Index files from the resourcelocation recursively and store the index at
	 * the indexRoot position
	 *
	 * @param indexRoot
	 * @param resourcelocation
	 * @throws IOException
	 * @throws ContentParserException
	 */
	public void index(String indexRoot, URI resourcelocation)
			throws IOException, ContentParserException {

		// get Lucene instances
		Directory dir = FSDirectory.getDirectory(indexRoot);
		Analyzer analyzer = new SimpleAnalyzer();
		//IndexWriter writer = new IndexWriter(dir, analyzer, true, MaxFieldLength.UNLIMITED);
		IndexWriter writer = new IndexWriter(dir, analyzer, true);

		long start = new Date().getTime();
		// start indexing
		index(writer, new File(resourcelocation));
		writer.optimize();
		writer.close();
		long end = new Date().getTime();

		stats(dir, start, end);

	}

	/**
	 * just print some stats
	 *
	 * @param dir
	 * @param start
	 * @param end
	 * @throws CorruptIndexException
	 * @throws IOException
	 */
	private void stats(Directory dir, long start, long end)
			throws CorruptIndexException, IOException {

		IndexReader reader = IndexReader.open(dir);
		logger.debug("Documents indexed: " + reader.numDocs());
		logger.debug("Total time: " + (end - start) + " ms");
		reader.close();
	}

	/**
	 * set the content parser
	 *
	 * @param contentParser
	 */
	public void setContentParser(ContentParser contentParser) {
		synchronized (contentParserAccess) {
			this.contentParser = contentParser;
		}

	}

	/**
	 * return the current content parser
	 *
	 * @return
	 */
	public ContentParser getContentParser() {
		return contentParser;
	}

	public static void main(String[] args) {
		Indexer indexer;

		try {
			// initialize indexer implementation
			indexer = new Indexer();

			// set our tika parser
			indexer.setContentParser(new TikaDocumentParser());

			// start indexing process
			URL test_documents = Indexer.class.getClassLoader().getResource("test-documents");
			logger.debug(test_documents);

			indexer.index("index",test_documents.toURI());
		} catch (Exception e) {
			logger.error(e);
		}
	}

}
