package de.acidum.tika;

import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;

import javax.imageio.ImageIO;
import javax.imageio.ImageReader;

import org.apache.commons.io.input.CloseShieldInputStream;
import org.apache.log4j.Logger;
import org.apache.tika.exception.TikaException;
import org.apache.tika.metadata.Metadata;
import org.apache.tika.parser.Parser;
import org.apache.tika.sax.XHTMLContentHandler;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

import com.drew.imaging.jpeg.JpegMetadataReader;
import com.drew.imaging.jpeg.JpegProcessingException;
import com.drew.metadata.Directory;
import com.drew.metadata.MetadataException;
import com.drew.metadata.Tag;

/**
 * This class implements a tika parser. To activate the parser we have to change
 * the tika-config.xml.
 *
 * Compared to the default Tika Image handling we read the Jpeg Exif data and
 * return these values as metadata for Lucene
 *
 * @author Christoph Hartmann
 *
 */
public class ImageParser implements Parser {

	Logger logger = Logger.getLogger(this.getClass());

	public void parse(InputStream stream, ContentHandler handler,
			Metadata metadata) throws IOException, SAXException, TikaException {

		String type = metadata.get(Metadata.CONTENT_TYPE);

		if (type != null) {
			// hey we get a jpeg lets read the exif
			if (type.equals("image/jpeg")) {
				extractJPEGMetaData(stream, metadata);
			}
			// if picture is unknown do the default tika handling
			else {

				Iterator<ImageReader> iterator = ImageIO
						.getImageReadersByMIMEType(type);
				if (iterator.hasNext()) {
					ImageReader reader = iterator.next();
					reader.setInput(ImageIO
							.createImageInputStream(new CloseShieldInputStream(
									stream)));
					metadata.set("height", Integer
							.toString(reader.getHeight(0)));
					metadata.set("width", Integer.toString(reader.getWidth(0)));
					reader.dispose();
				}
			}
		}

		XHTMLContentHandler xhtml = new XHTMLContentHandler(handler, metadata);
		xhtml.startDocument();
		xhtml.endDocument();
	}

	/**
	 * get additional metadata for jpeg files
	 *
	 * @param inputStream
	 * @param tikaMetaData
	 */
	private void extractJPEGMetaData(InputStream inputStream,
			Metadata tikaMetaData) {

		// read the exif meta data
		com.drew.metadata.Metadata jpegMetaData;
		try {
			jpegMetaData = JpegMetadataReader.readMetadata(inputStream);

			// iterate through metadata directories
			Iterator<?> directories = jpegMetaData.getDirectoryIterator();
			while (directories.hasNext()) {
				Directory directory = (Directory) directories.next();
				// iterate through tags and print to System.out
				Iterator<?> tags = directory.getTagIterator();
				while (tags.hasNext()) {
					Tag tag = (Tag) tags.next();

					try {
						tikaMetaData.set(tag.getDirectoryName() + "."
								+ tag.getTagName(), tag.getDescription());
						logger.debug(tag.getDirectoryName() + "."
								+ tag.getTagName() + " -> "
								+ tag.getDescription());
					} catch (MetadataException e) {
						logger.error(e);
					}
				}
			}

		} catch (JpegProcessingException e) {
			logger.error(e);
		}
	}
}